<?php
/**
 * Created by PhpStorm.
 * User: pengfei@yidian-inc.com
 * Date: 2021/9/3 2:23 下午
 */

use App\Base\Job;
use Api\PhpServices\Idgen\Idgen;
use App\Models\marketing\mysql\ColonelDistributorColonel;
use App\Models\marketing\mysql\ColonelDistributorConfig;
use App\Models\marketing\mysql\ColonelDistributorInviteOrderNum;
use App\Models\marketing\mysql\ColonelDistributorPayInfo;
use App\Exception\custom\InterfaceException;
use Api\PhpUtils\Http\HttpUtil;
use App\Models\user\mysql\UserWechatBind;
use App\Services\marketing\PindanActivityColonelConfigService;
use Api\PhpUtils\Log\JobLog;

class ColonelorderController extends Job
{
    const REWARD_UNIT = 100;  // 奖励单位: 分
    const INVITE_NEW_USER_REWARD = 3 * self::REWARD_UNIT; // 邀请新用户奖励
    const DEFAULT_USER_BATCH_SIZE = 100; // 批量处理数据条数

    public $colonelConfigList; // 活动配置
    public $settlementDate;    // 活动结算日
    public $startTime;         // 活动结算日-开始时间
    public $endTime;           // 活动结算日-结束时间

    /**
     * Notes: 初始化-读取配置
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:13 下午
     */
    public function init()
    {
        parent::init();
        $timeRange = PindanActivityColonelConfigService::getSettlementTimeRange();
        $this->settlementDate = $timeRange['date'];
        $this->startTime = $timeRange['start_time'];
        $this->endTime = $timeRange['end_time'];
        $this->colonelConfigList = $this->getColonelConfig($this->settlementDate);
    }

    /**
     * Notes: 计算奖励脚本
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:05 下午
     * @throws InterfaceException
     */
    public function indexAction()
    {
        $this->loggerInfo('start');
        if ($userIdList = ColonelDistributorColonel::getColonelUserIds()) {
            $userIdList = array_chunk($userIdList, self::DEFAULT_USER_BATCH_SIZE);
            $colonelPayInfoData = [];
            foreach ($userIdList as $userIds) {
                $this->getInviteOrderData($userIds, $colonelPayInfoData);   // 完成目标单数
                $this->getInviteNewUserData($userIds, $colonelPayInfoData); // 邀请新用户奖励
            }
            $this->handleSavePayInfo($colonelPayInfoData);
        }
        $this->loggerInfo('end');
    }

    /**
     * Notes: 保存奖励信息
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:34 下午
     * @param $colonelPayInfoData
     */
    private function handleSavePayInfo($colonelPayInfoData)
    {
        $this->loggerInfo('colonelPayInfoCount=' . count($colonelPayInfoData));
        if (!empty($colonelPayInfoData)) {
            $data = $this->getFormatPayInfoData($colonelPayInfoData);
            try {
                ColonelDistributorPayInfo::beginTransaction();
                if (!$this->addColonelDistributorPayInfo($data)) {
                    throw new Exception(sprintf("写入错误参数 %s", json_encode($data)));
                }
                if (!ColonelDistributorPayInfo::commit()) {
                    throw new Exception(sprintf("事务提交失败 %s", json_encode($data)));
                }
                $this->loggerInfo('success');
            } catch (Exception $e) {
                ColonelDistributorPayInfo::rollback();
                $this->loggerError('error=' . $e->getMessage());
            }
        }
    }

    /**
     * Notes: 获取新用户奖励
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:47 下午
     * @param $userIds
     * @param $colonelPayInfoData
     * @throws InterfaceException
     */
    private function getInviteNewUserData($userIds, &$colonelPayInfoData)
    {
        if ($inviteNewUserList = $this->getInviteNewUserList($userIds, $this->startTime, $this->endTime)) {
            $userNickList = $this->getUserList(array_column($inviteNewUserList, 'user_id'));
            foreach ($inviteNewUserList as $newUser) {
                $colonelPayInfoData[] = [
                    'date'             => $this->settlementDate,
                    'type'             => ColonelDistributorPayInfo::TYPE_NEW_USER,
                    'reward'           => self::INVITE_NEW_USER_REWARD,
                    'finish_num'       => 1,
                    'invite_user_id'   => $newUser['user_id'],
                    'invite_user_nick' => empty($userNickList[$newUser['user_id']]['user_nick']) ? '' : $userNickList[$newUser['user_id']]['user_nick'],
                    'colonel_user_id'  => $newUser['inviter_user_id'],
                ];
            }
        }
    }

    /**
     * Notes: 获取完成目标单数奖励
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:47 下午
     * @param $userIds
     * @param $colonelPayInfoData
     */
    private function getInviteOrderData($userIds, &$colonelPayInfoData)
    {
        if (!empty($this->colonelConfigList)) {
            $inviteOrderNumList = $this->getInviteOrderNum($userIds, $this->settlementDate);
            foreach ($inviteOrderNumList as $orderNum) {
                foreach ($this->colonelConfigList as $config) {
                    if ($orderNum['num'] >= $config['assess_order_num']) {
                        $colonelPayInfoData[] = [
                            'date'             => $this->settlementDate,
                            'type'             => ColonelDistributorPayInfo::TYPE_FINISH_TARGET,
                            'reward'           => $config['reward_amount'] * self::REWARD_UNIT,
                            'finish_num'       => $config['assess_order_num'],
                            'invite_user_id'   => 0,
                            'invite_user_nick' => '',
                            'colonel_user_id'  => $orderNum['colonel_user_id'],
                        ];
                    }
                }
            }
        }
    }

    /**
     * 团长任务配置
     * @param $settlementDate
     * @return array
     */
    private function getColonelConfig($settlementDate): array
    {
        $config = ColonelDistributorConfig::getRecord([
            'date[<=]' => $settlementDate,
            'type'     => ColonelDistributorConfig::TYPE_COLONEL,
            'ORDER'    => ['date' => 'DESC'],
            'LIMIT'    => 1
        ]);
        $configData = [];
        if (!empty($config['data'])) {
            $configData = json_decode($config['data'], true);
        }
        return $configData;
    }

    /**
     * 获取order num
     * @param $colonelUserIds
     * @param $date
     * @return array
     */
    private function getInviteOrderNum($colonelUserIds, $date): array
    {
        return ColonelDistributorInviteOrderNum::getRecords(
            [
                'colonel_user_id' => $colonelUserIds,
                'date'            => $date,
            ],
            ['colonel_user_id', 'num']
        );
    }

    /**
     * Notes: 获取有效新用户信息
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:03 下午
     * @param $userIds
     * @param $startTime
     * @param $endTime
     * @return array
     * @throws InterfaceException
     */
    private function getInviteNewUserList($userIds, $startTime, $endTime): array
    {
        $url = config('interface', 'coupon.background.wx_invite_relation_list_by_user_ids');
        if (empty($url)) {
            throw new InterfaceException(['cus' => 0]);
        }

        $res = HttpUtil::get($url,
            ['user_ids' => $userIds, 'start_time' => $startTime, 'end_time' => $endTime]);
        if (!empty($res['response']['result'])) {
            return array_values(array_column($res['response']['result'], null, 'user_id'));
        }
        return [];
    }

    /**
     * Notes: 获取pay_info数据
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:07 下午
     * @param $data
     * @return array
     */
    private function getFormatPayInfoData($data): array
    {
        $count = count($data);
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [
            [
                'type'   => 'goods',
                'number' => 00,
                'count'  => $count
            ]
        ]);
        $ids = $res['id_datetime']['goods'] ?? [];
        foreach ($data as $key => $item) {
            $data[$key]['pay_info_trade_id'] = $ids[$key];
        }
        return $data;
    }

    /**
     * Notes: 用户信息，获取昵称
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:09 下午
     * @param $userIds
     * @return array
     */
    private function getUserList($userIds): array
    {
        $list = UserWechatBind::select(['user_id', 'user_nick'], ['user_id' => $userIds], []);
        return array_column((array)$list, null, 'user_id');
    }

    /**
     * Notes: 写入数据
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:10 下午
     * @param $data
     * @return bool
     */
    private function addColonelDistributorPayInfo($data): bool
    {
        return (int)ColonelDistributorPayInfo::save($data);
    }

    /**
     * Notes: 记录 info 日志
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:10 下午
     * @param $log
     */
    private function loggerInfo($log)
    {
        JobLog::getInstance()->info('colonel_distributor_order', $log)->output();
    }

    /**
     * Notes: 记录 error 日志
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:10 下午
     * @param $log
     */
    private function loggerError($log)
    {
        JobLog::getInstance()->error('colonel_distributor_order', $log, null, 'bp-server@yidian-inc.com')->output();
    }
}
