<?php
/**
 * Created by PhpStorm.
 * User: pengfei@yidian-inc.com
 * Date: 2021/9/3 2:23 下午
 */

use App\Base\Job;
use App\Models\marketing\mysql\ColonelDistributorPayInfo;
use App\Exception\custom\InterfaceException;
use Api\PhpUtils\Http\HttpUtil;
use Api\PhpUtils\Log\JobLog;

class ColonelwalletController extends Job
{
    const CAPITAL_POOL_ID = 2;   // 资金池id
    const CAPITAL_POOL_OP = 200; //200：团长奖励
    const WALLET_SERVICE_NAME = 11; // 10生活圈优惠券，11营销活动
    const WALLET_SOURCE_NAME = 1; //使用方名，1生活圈，2主端

    /**
     * Notes: 奖励结算脚本
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 6:03 下午
     * @throws InterfaceException
     */
    public function indexAction()
    {
        $this->loggerInfo('start');
        $payInfoList = $this->getPayInfoList();
        $this->loggerInfo('payInfoCount=' . count($payInfoList));
        if (!empty($payInfoList)) {
            if ($this->deductingInventory($payInfoList)) {
                if ($this->toWallet($payInfoList)) {
                    if ($this->handlePayInfoStatus(array_column($payInfoList, 'pay_info_id'))) {
                        $this->loggerInfo('success');
                    } else {
                        $this->loggerError('更新状态失败');
                    }
                }
            }
        }
        $this->loggerInfo('end');
    }

    /**
     * Notes: 结算写入钱包
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 6:04 下午
     * @param $payInfoList
     * @return bool
     * @throws InterfaceException
     */
    private function toWallet($payInfoList): bool
    {
        if (!$url = config('interface', 'wallet.account.transfer')) {
            throw new InterfaceException(['cus' => 0]);
        }
        $httpParams = self::getWalletHttpParams();
        $list = [];
        foreach ($payInfoList as $payInfo) {
            $list[] = [
                'user_id'             => $payInfo['colonel_user_id'],
                'service_name'        => self::WALLET_SERVICE_NAME,
                'source_name'         => self::WALLET_SOURCE_NAME,
                'third_order_id'      => $payInfo['pay_info_trade_id'],
                'third_order_id_type' => $payInfo['type'],
                'third_order_desc'    => self::getThirdOrderDesc($payInfo),
                'amount'              => $payInfo['reward'],
                'extra'               => $payInfo
            ];
        }
        $httpParams['list'] = $list;
        $response = HttpUtil::post($url, $httpParams, 100000, 3);
        $this->loggerInfo("url=$url,response:" . json_encode($response));
        if (!empty($response['response']['result']) && $response['code'] == 0) {
            $this->loggerInfo('调用钱包服务成功,count=' . count($payInfoList));
            return true;
        } else {
            $this->loggerError('调用钱包服务失败,httpParams=' . json_encode($httpParams));
            return false;
        }
    }


    /**
     * Notes: 资金池授予用户奖励
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 6:04 下午
     * @param $payInfoList
     * @return bool
     * @throws InterfaceException
     */
    private function deductingInventory($payInfoList): bool
    {
        if (!$url = config('interface', 'coupon.capitalpool.grant_user_amount')) {
            throw new InterfaceException(['cus' => 0]);
        }
        $httpParams = [
            'capital_pool_id' => self::CAPITAL_POOL_ID
        ];
        $body = [];
        foreach ($payInfoList as $payInfo) {
            $body[] = [
                'op'                  => self::CAPITAL_POOL_OP,
                'user_id'             => $payInfo['colonel_user_id'],
                'amount'              => $payInfo['reward'],
                'third_order_id'      => $payInfo['pay_info_trade_id'],
                'third_order_id_type' => $payInfo['type'],
                'third_order_desc'    => self::getThirdOrderDesc($payInfo),
            ];
        }
        $httpParams['body'] = $body;
        $res = HttpUtil::post($url, $httpParams, 100000, 3);
        $this->loggerInfo("url=$url,response:" . json_encode($res));
        if ($res['code'] == 0 && isset($res['response']['result'])) {
            $this->loggerInfo('调用资金池服务成功,count=' . count($payInfoList));
            return true;
        } else {
            $this->loggerError('调用资金池服务失败,httpParams=' . json_encode($httpParams));
            return false;
        }
    }

    /**
     * Notes: 获取钱包服务请求参数
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 7:34 下午
     * @return array
     */
    private function getWalletHttpParams(): array
    {
        // TODO 此处看后期业务发展 看是否需要封装WalletSdk
        $date = date('Y-m-d H:i:s');
        return [
            'transfer_service_id' => appConfig('colonel.wallet.transfer_service_id'),
            'sign'                => self::getWalletHttpSign($date),
            'time'                => $date
        ];
    }

    /**
     * Notes: 获取钱包签名
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 7:34 下午
     * @param $date
     * @return string
     */
    private function getWalletHttpSign($date): string
    {
        return md5($date . appConfig('colonel.wallet.transfer_service_secret'));
    }

    /**
     * Notes: 获取待结算数据
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 7:23 下午
     * @return array
     */
    private function getPayInfoList(): array
    {
        return ColonelDistributorPayInfo::getRecords(['status' => ColonelDistributorPayInfo::STATUS_NOT_USE]);
    }

    /**
     * Notes: 获取描述信息
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 6:10 下午
     * @param $payInfo
     * @return string
     */
    private function getThirdOrderDesc($payInfo): string
    {
        $desc = '';
        if ($payInfo['type'] == ColonelDistributorPayInfo::TYPE_NEW_USER) {
            $desc = "邀请新用户（" . $payInfo["invite_user_nick"] . "）完成下单奖励";
        } elseif ($payInfo['type'] == ColonelDistributorPayInfo::TYPE_FINISH_TARGET) {
            $desc = "完成" . $payInfo["finish_num"] . "单用户下单奖励";
        }
        return $desc;
    }

    /**
     * Notes: 修改状态
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 6:09 下午
     * @param array $payInfoIds
     * @return bool
     */
    private function handlePayInfoStatus(array $payInfoIds): bool
    {
        return ColonelDistributorPayInfo::updateRecord(['status' => ColonelDistributorPayInfo::STATUS_USE],
            ['pay_info_id' => $payInfoIds]);
    }

    /**
     * Notes: 记录 info 日志
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:10 下午
     * @param $log
     */
    private function loggerInfo($log)
    {
        JobLog::getInstance()->info('colonel_distributor_wallet', $log, false)->output();
    }

    /**
     * Notes: 记录 error 日志
     * User: pengfei@yidian-inc.com
     * Date: 2021/9/3 4:10 下午
     * @param $log
     */
    private function loggerError($log)
    {
        JobLog::getInstance()->error('colonel_distributor_wallet', $log, null, 'bp-server@yidian-inc.com')->output();
    }
}
